#ifndef __PID_H
#define __PID_H

#include "mytypes.h"

//Constants used in some of the functions below
#define AUTOMATIC	1
#define MANUAL	0
#define DIRECT  0
#define REVERSE  1

typedef struct PID {
	u8 controllerDirection:1;

	float kp;                  	// * (P)roportional Tuning Parameter
	float ki;                  	// * (I)ntegral Tuning Parameter
	float kd;                  	// * (D)erivative Tuning Parameter


	float input;     
	float output;    
										
	float ITerm, lastInput;

	u16 	sampleTime;					// in milliseconds	
	float outMin, outMax;
	
} PID_t; 

//commonly used functions **************************************************************************
void PID_Create(PID_t* ctx, float, float, float, u16 sampleTime, u8 ControllerDirection);			// * constructor.  links the PID to the Input, Output, and 
																														//   Setpoint.  Initial tuning parameters are also set here

float PID_Compute(PID_t* ctx, float input, float setpoint);   				// * performs the PID calculation.  it should be
																					//   called every time loop() cycles. ON/OFF and
																					//   calculation frequency can be set using SetMode
																					//   SetSampleTime respectively

void PID_SetOutputLimits(PID_t* ctx,float, float); 		// clamps the output to a specific range. 0-255 by default, but
																					// it's likely the user will want to change this depending on
																					// the application

// available but not commonly used functions ********************************************************
void PID_SetTunings(PID_t* ctx,float, float,       		// * While most users will set the tunings once in the 
															float);         	//   constructor, this function gives the user the option
																					//   of changing tunings during runtime for Adaptive control
															
void PID_SetControllerDirection(PID_t* ctx,int);	  				// * Sets the Direction, or "Action" of the controller. DIRECT
																					//   means the output will increase when error is positive. REVERSE
																					//   means the opposite.  it's very unlikely that this will be needed
																					//   once it is set in the constructor.
	
void PID_SetSampleTime(PID_t* ctx,int); 						// * sets the frequency, in Milliseconds, with which 
																					//   the PID calculation is performed.  default is 100

//Display functions ****************************************************************
float PID_GetKp(PID_t* ctx);						  				// These functions query the pid for interal values.
float PID_GetKi(PID_t* ctx);						  				//  they were created mainly for the pid front-end,
float PID_GetKd(PID_t* ctx);						  				// where it's important to know what is actually 
int PID_GetMode(PID_t* ctx);						  					//  inside the PID.
int PID_GetDirection(PID_t* ctx);					  					//

void PID_Initialize(PID_t* ctx);


#endif